--[[
		Vector Class
		Copyright (c) 2012 Roland Yonaba
--]]

--[[
Copyright (c) 2012 Roland Yonaba

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
--]]

local setmetatable, getmetatable = setmetatable, getmetatable
local type = type
local assert = assert
local require = require
local cos, sin, sqrt = math.cos,math.sin,math.sqrt
local format = string.format

local Vector = {}
Vector.__index = Vector

-- Creates a new vector
local function new(x,y)
	local v = {x = x or 0, y = y or 0}
	return setmetatable(v,Vector)
end

-- Tests if vectors a and b are component-by-component equals
function Vector.__eq(a,b)
	return ((a.x == b.x) and (a.y == b.y))
end

-- Returns the sum of a and
function Vector.__add(a,b)
	return new(a.x+b.x,a.y+b.y)
end

-- Returns the difference between a and b
function Vector.__sub(a,b)
	return new(a.x-b.x,a.y-b.y)
end

-- Returns the product between a and b
function Vector.__mul(a,b)
	if type(a) == 'number' then
		return new (a*b.x,a*b.y)
	elseif type(b) == 'number' then
		return new(b*a.x,b*a.y)
	end
	return new(a.x*b.x,a.y*b.y)
end

-- Returns the quotient of a / b
function Vector.__div(a,b)
	if type(a) == 'number' then
		assert(a ~= 0, 'Cannot divide by zero')
		return new ((1/a)*b.x,(1/a)*b.y)
	elseif type(b) == 'number' then
		assert(b ~= 0, 'Cannot divide by zero')
		return new ((1/b)*a.x,(1/b)*a.y)
	end
	return new(a.x/b.x,a.y/b.y)
end

-- Returns vector pow a
function Vector.__pow(a,b)
	if type(a) == 'number' then
		return new (b.x^a, b.y^a)
	elseif type(b) == 'number' then
		return new (a.x^b, a.y^b)
	end
end

-- Returns if a < b
function Vector.__lt(a,b)
	return (a.x < b.x) and (a.y < b.y)
end

-- Returns if a <= b
function Vector.__le(a,b)
	return (a.x <= b.x) and (a.y <= b.y)
end

-- Returns the unary vector
function Vector.__unm()
	return new(-self.x,-self.y)
end

-- Returns the clone vector
function Vector:clone()
	return new(self.x,self.y)
end

-- Returns if self is a vector
function Vector:isVector()
	return getmetatable(self) == Vector
end

-- Returns determinant(self, a)
function Vector:determinant(a)
	assert(p:isVector(),'Argument must be a vector')
	return (self.x*a.y-self.y*a*x)
end

-- Returns a vector perpendicular to self
function Vector:perpendicular()
	return new (-self.y,self.x)
end

-- Returns a vector rotated from phi
function Vector:rotate(phi)
	assert(phi, 'Argument mst be an angle in radians')
	local cosine,sine = cos(phi), sin(phi)
	return new(cosine * self.x - sine * self.y, sine * self.x + cosine * self.y)
end

-- Clamps a vector
function Vector:clamp(v_min,v_max)
	if self < v_min then return v_min end
	if v_max < self then return v_max end
	return self
end

-- Returns self magnitude
function Vector:magnitude()
	return sqrt(self.x^2+self.y^2)
end

-- Returns self squared magnitude
function Vector:squaredMagnitude()
	return (self.x^2+self.y^2)
end

-- Returns a normalized vector
function Vector:norm()
	local l = self:magnitude()
	assert (l > 0, 'Cannot normalize a Zero-Vector')
	return new(self.x/l,self.y/l)
end

-- Sets Vector to a Zero-Vector
function Vector:clear()
	self.x,self.y = 0,0
end

-- Printing a vector
function Vector.__tostring(self,name)
	return ("%s : { x = %.2f , y = %.2f }"):format(name or 'vector',self.x,self.y)
end


return setmetatable(Vector,{__call = function (self, ...) return new (...) end})


